// 包结构
import 'dart:typed_data';
import 'crc.dart';

class Packet {
  PacketHeader header; // 包头
  Uint8List data; // 数据
  int crc; // 2字节，CRC校验码

  Packet(this.header, this.data, this.crc);

  PacketType get type => header.type;

  /// 将数据包序列化为字节数组（包头+数据+CRC）
  Uint8List toBytes() {
    // 创建包头字节（5字节）
    Uint8List headerBytes = Uint8List(5);
    headerBytes[0] = header.type.value; // 类型
    headerBytes[1] = (header.seqNum >> 8) & 0xFF; // 序列号高字节
    headerBytes[2] = header.seqNum & 0xFF; // 序列号低字节
    headerBytes[3] = (header.dataLen >> 8) & 0xFF; // 数据长度高字节
    headerBytes[4] = header.dataLen & 0xFF; // 数据长度低字节

    // 创建CRC字节（2字节）
    Uint8List crcBytes = Uint8List(2);
    crcBytes[0] = (crc >> 8) & 0xFF; // CRC高字节
    crcBytes[1] = crc & 0xFF; // CRC低字节

    // 合并所有部分：包头 + 数据 + CRC
    return Uint8List.fromList([...headerBytes, ...data, ...crcBytes]);
  }

  @override
  String toString() {
    // 包类型、序列号、数据长度、CRC
    return 'Packet{type=${header.type}, seqNum=${header.seqNum}, dataLen=${header.dataLen}, crc=${crc.toRadixString(16)},\nallData=${toBytes().map((e) => e.toRadixString(16)).join(",")}}';
  }
}

// 包头结构
class PacketHeader {
  PacketType type; // 1字节，包类型
  int seqNum; // 2字节，序列号
  int dataLen; // 2字节，数据长度

  PacketHeader(this.type, this.seqNum, this.dataLen);
}

/// 数据包类型
enum PacketType {
  // 用于传输开始时发送，包含文件大小和分块大小等信息
  start(0x01, '开始包'),
  // 用于传输实际的文件数据块
  data(0x02, '数据包'),
  // 用于确认某个数据包已正确接收
  ack(0x03, '确认包'),
  // 用于指示某个数据包接收失败，请求重传
  nak(0x04, '重传请求包'),
  // 用于标识文件传输结束
  end(0x05, '结束包'),
  // 错误
  error(0x06, '错误包'),
  ;

  const PacketType(this.value, this.label);

  final int value;
  final String label;

  static PacketType fromValue(int value) {
    switch (value) {
      case 0x01:
        return start;
      case 0x02:
        return data;
      case 0x03:
        return ack;
      case 0x04:
        return nak;
      case 0x05:
        return end;
      case 0x06:
        return error;
      //     默认为error
      default:
        return error;
    }
  }
}

// 打包函数
Packet packPacket(PacketType type, int seqNum, Uint8List? data) {
  int dataLen = data?.length ?? 0;
  PacketHeader header = PacketHeader(type, seqNum, dataLen);
  Uint8List packetData = data ?? Uint8List(0);

  // 创建包头字节（5字节）
  Uint8List headerBytes = Uint8List(5);
  headerBytes[0] = header.type.value; // 类型
  headerBytes[1] = (header.seqNum >> 8) & 0xFF; // 序列号高字节
  headerBytes[2] = header.seqNum & 0xFF; // 序列号低字节
  headerBytes[3] = (header.dataLen >> 8) & 0xFF; // 数据长度高字节
  headerBytes[4] = header.dataLen & 0xFF; // 数据长度低字节

  // 合并包头和数据
  Uint8List fullData = Uint8List.fromList(headerBytes + packetData);

  // 计算CRC
  int crc = crc16(fullData);

  return Packet(header, packetData, crc);
}
